@@ -1,12 +1,21 @@
 """Verify that warnings are issued for global statements following use."""
 
-from test.test_support import run_unittest, check_syntax_error
+from test.support import run_unittest, check_syntax_error, check_warnings
 import unittest
 import warnings
 
 
 class GlobalTests(unittest.TestCase):
 
+    def setUp(self):
+        self._warnings_manager = check_warnings()
+        self._warnings_manager.__enter__()
+        warnings.filterwarnings("error", module="<test string>")
+
+    def tearDown(self):
+        self._warnings_manager.__exit__(None, None, None)
+
+
     def test1(self):
         prog_text_1 = """\
 def wrong1():
@@ -15,24 +24,24 @@ def wrong1():
     global a
     global b
 """
-        check_syntax_error(self, prog_text_1)
+        check_syntax_error(self, prog_text_1, lineno=4, offset=4)
 
     def test2(self):
         prog_text_2 = """\
 def wrong2():
-    print x
+    print(x)
     global x
 """
-        check_syntax_error(self, prog_text_2)
+        check_syntax_error(self, prog_text_2, lineno=3, offset=4)
 
     def test3(self):
         prog_text_3 = """\
 def wrong3():
-    print x
+    print(x)
     x = 2
     global x
 """
-        check_syntax_error(self, prog_text_3)
+        check_syntax_error(self, prog_text_3, lineno=4, offset=4)
 
     def test4(self):
         prog_text_4 = """\
